#include "jukebox.h"

void lcd_write(int byte, int data)
{
   int i;
   char on,off;

   PBDR &= ~CS1; /* enable lcd chip select */

   if ( data ) {
      on=~(SDA|SCK);
      off=SCK|DC;
   }
   else {
      on=~(SDA|SCK|DC);
      off=SCK;
   }
   /* clock out each bit, MSB first */
   for (i=0x80;i;i>>=1)
   {
      PBDR &= on;
      if (i & byte)
         PBDR |= SDA;
      PBDR |= off;
   }

   PBDR |= CS1; /* disable lcd chip select */
}

void lcd_printxy( char x, char y, unsigned char* string, int len )
{
   int i;
   lcd_write(LCD_GOTOXY+y*16+x,0);
   for (i=0; string[i] && i<len; i++)
      lcd_write(ascii2lcd[string[i]],1);
}

void lcd_writexy( char x, char y, int byte)
{
//   int i;
   lcd_write(LCD_GOTOXY+y*16+x,0);
//   for (i=0; string[i] && i<len; i++)
//      lcd_write(string[i],1);
      lcd_write(byte,1);
}

#ifndef OLD_ROM
// this function depends on LCD_SETICONRAM which is undefined for older units
void lcd_seticon( char addr, char mask)
{
   lcd_write(LCD_SETICONRAM + addr,0);
   lcd_write( mask,1);
}
int mlevs[] = { 0xA0, 0x80, 0x81, 0x82, 0x83, 0x84, 0x85, 0x86 };
#else
int mlevs[] = { 0x24, 0x15, 0xDF, 0xE0, 0xE1, 0xE2, 0xE3, 0xEC };
#endif

const char hextab[] = "0123456789ABCDEF";

int Sci1Err( void)
{
        lcd_write(0x10,0);
        lcd_printxy(0,0,"SCI1ERR",7);
}       

int Sci1Rx( void)
{
        lcd_write(0x10,0);
        lcd_printxy(0,0,"SCI1RX",6);
}

const void* vectors[] __attribute__ ((section (".vectors"))) =
{
   main,        /*00 Power-on reset */
   stack,       /*01 Power-on reset (stack pointer) */
   main,        /*02 Manual reset */
   stack,       /*03 Manual reset (stack pointer) */
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, // 04
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, // 24
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, // 44
   0,0,0,0,             // 64
//   Sci1Err,                             // 68 = 104
//   Sci1Rx,                              // 69 = 105
}; 

char *itoa(int i)
{
	/* 10 digits + 1 sign + 1 trailing nul */
	static char buf[12];
	char *pos = buf + sizeof(buf) - 1;
	unsigned int u;
	int negative = 0;

	if (i < 0) {
		negative = 1;
		u = ((unsigned int)(-(1+i))) + 1;
	} else {
		u = i;
	}

	*pos = 0;

	do {
		*--pos = '0' + (u % 10);
		u /= 10;
	} while (u);

	if (negative) {
		*--pos = '-';
	}

	return pos;
}

void lcd_graphx(int x, int y) {
        int lev0, lev1;
        if (y>7) {
                lev0=y-7;
                //lev1=y-lev0;
                lev1=7;
        } else {
                lev0=0;
                lev1=y;
        }
        lcd_writexy(x,1, mlevs[lev1]);
        lcd_writexy(x,0, mlevs[lev0]);
}
