#include "jukebox.h"

/* uncommont the line in jukebox.h for use with Really Old units: */

#ifdef OLD_ROM

#define B0 0x8B
#define BR 0x8B
#define BT 0xDB
#define FE 0x24
#define AU 0x86
#define AD 0x87
#define AL 0x40
#define AR 0x42
#define EX 0x2E

static const unsigned char player[] = { 0x88, 0x86, 0x89, 0x87 };
#define PLAY 0x89

#else

#define B0 0xf0
#define BR 0xfd
#define BT 0xf7
#define FE 0xA0
#define AU 0x18
#define AD 0x19
#define AL 0x7F
#define AR 0x7E
#define EX 0x90

static const unsigned char player[] = { 0x11, 0x1F, 0x10, 0x1E };
#define PLAY 0x10

#endif

// This is the map
// anything over B0 is considered solid for collision detection
// anything under can be walked through
// EX marks map end...
// this will all be changed once i figure out custom icons...
// FIXME: make maps dynamically generated, or at least work out a level system
static const unsigned char mapa[][25] = {
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR },
	{ BR,FE,BR,BR,BR,BR,FE,FE,FE,FE,FE,FE,FE,FE,BR,FE,FE,FE,FE,FE,FE,BR,EX,BR },
	{ BR,FE,FE,FE,FE,FE,FE,BR,FE,FE,BT,BR,BT,FE,FE,FE,BT,BR,BR,BT,FE,BR,FE,BR },
	{ BR,BR,BR,BR,FE,FE,FE,FE,BR,FE,FE,FE,FE,FE,BR,FE,FE,FE,FE,FE,FE,BR,FE,BR },
	{ BR,FE,FE,FE,FE,BR,BR,FE,FE,FE,BT,BT,BR,FE,FE,FE,BR,BR,BR,BR,FE,BR,FE,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,BR,BR,BR,FE,BR,BR,FE,FE,FE,FE,BR,FE,FE,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,BR,FE,FE,FE,BR,BR,FE,BR,BR,FE,BR,FE,BR,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,BR,FE,BR,BR,BR,BR,FE,BR,FE,FE,BR,FE,BR,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,BR,FE,FE,FE,BR,FE,FE,BR,FE,BR,BR,FE,FE,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,FE,BR,BR,FE,FE,FE,FE,BR,FE,BR,BR,BR,FE,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,FE,FE,BR,BR,BR,BR,BR,FE,FE,FE,FE,AU,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,FE,FE,FE,FE,FE,BR,BR,BR,BR,BR,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,FE,FE,FE,FE,FE,AL,BR },
	{ BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR,BR },
};
int mapasize=sizeof(mapa[0]);

// Prototypes...
void choice(void);
void game(void);
void shutoff(void);
void intro(void);
void lcd_printmap(char y, char xoff, unsigned char* string, int len);

// Called when they hold the stop button
void shutoff(void) {
	lcd_printxy(0,0,"...Shutting",11);
	lcd_printxy(0,1,"    Down...",11);
}

// prints string directly to LCD without ascii charset conversion
// y is the line to print on, xoff is how much to offset on the x axis
// 	(for horizontal scrolling)
// string is the string to be printed, len is it's length
void lcd_printmap(char y, char xoff, unsigned char* string, int len) {
	int i;
	lcd_write(LCD_GOTOXY+y*16,0);
	for (i=0; string[i] && i<len; i++)
		lcd_write(string[i+xoff],1);
}

// called when we start...
void intro(void) {
	int j;
	lcd_printxy(0,0,"Welcome to ",11);
	lcd_printxy(0,1,"the GameBox",11);
	for (j=0; j<3000000; j++);
}

// gives choice of play or quit, calls game to play which comes back into the
// loop when finished, calls shutoff() and breaks if stop is held
void choice(void) {
	int t;
	while(1) {
		lcd_printxy(0,0," ? to play ",11);
		lcd_printxy(0,1,"off to quit",11);
		lcd_writexy(1,0,PLAY);
		if (KEY_PLAY) game();
		if (KEY_OFF) {
			shutoff();
			break;
		}
		for (t=0; t<150000;t++);
	}
}

// the actual game loop
void game(void) {
	int x=1,y=1,j,led=0,xo=1,yo=1;
	int mapx=0,mapy=0,dir=0,pdir=2;
	while(1) {
		for (j=0; j<150000; j++);
		// we set the direction even if we can't move that way
		//   so if you're stuck against a wall and try to move
		//   towards it, it shows it to you.
		if (KEY_PLAY) { 
			if (mapa[y-1][x]<=B0) y--;
			dir=1;
			pdir=1;
		}
		if (KEY_STOP) {
			if (mapa[y+1][x]<=B0) y++;
			dir=0;
			pdir=3;
		}
		if (KEY_MINUS) {
			if (mapa[y][x-1]<=B0) x--;
			pdir=0;
		}
		if (KEY_PLUS) {
			if (mapa[y][x+1]<=B0) x++;
			pdir=2;
		}

		if ((x>5) && (x<(mapasize-6))) {
			xo=5;
			mapx=x-5;
		} else { xo=x; }
		if (x<=5) {
			mapx=0;
		}
		if (x>=(mapasize-6)) {
			xo=x-(mapasize-12);
			mapx=mapasize-12;
		}
		if (dir>=1) {
			yo=1;
			//mapy=y-1;
			mapy=y-2;
		}
		if (dir<1) {
			yo=0;
			mapy=y;
		}
		lcd_printmap(0,mapx,mapa[mapy+dir],mapasize);
		lcd_printmap(1,mapx,mapa[mapy+1+dir],mapasize);
		lcd_writexy(xo,yo,player[pdir]);
		// disregard: sleep was here
		PBDR ^= 0x40; /* toggle LED (PB6) */
		if (led==0) { led=1; } else { led=0; }
		if (mapa[y][x] == EX) {
			if (led==1) {
				PBDR ^= 0x40;
				led=0;
			}
			lcd_printxy(0,0,"You  Win!!!",11);
			lcd_printxy(0,1,"   YAY!!   ",11);
			for (j=0; j<3000000; j++);
			break;
		}
		if (KEY_OFF) {
			shutoff();
			break;
		}
	}
}

// just our main(), calls intro and the choice loop
int main(void) {
	intro();
	choice();
}
